/**
 * This file is part of
 * 
 * Repast Simphony MySQL Database Outputter
 *
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter.engine;



import java.io.File;

import repast.simphony.engine.controller.ControllerActionConstants;
import repast.simphony.engine.environment.ControllerAction;
import repast.simphony.scenario.AbstractDescriptorControllerActionIO;
import repast.simphony.scenario.ActionLoader;
import repast.simphony.scenario.DescriptorActionLoader;



/**
 * repast.simphony.data.db
 * 
 * The {@link ControllerAction} specified in the plug-in definition.
 * Instantiates a DbActionLoader that instantiates a {@link DbOutputterComponentControllerAction}.
 * 
 * @author Sascha Holzhauer
 * @date 13.04.2010
 * 
 * suppress warning: Parameters for DbOutputterDescriptorAction may not be defined since
 * repast.simphony.scenario.AbstractDescriptorControllerActionIO
 * expects DbOutputterDescriptorAction.class as constructor parameter.
 */
@SuppressWarnings("rawtypes")
public class DbOutputterControllerActionIO extends
		AbstractDescriptorControllerActionIO<DbOutputterComponentControllerAction, 
			DbOutputterDescriptor> {

	/**
	 * Calls super-constructor
	 */
	public DbOutputterControllerActionIO() {
		super(DbOutputterComponentControllerAction.class, DbOutputterDescriptor.class);
	}

	/**
	 * repast.simphony.data.db
	 *
	 * @author Sascha Holzhauer
	 * @date 14.10.2010 
	 *
	 */
	public static class DbActionLoader extends DescriptorActionLoader<DbOutputterDescriptor> {

		/**
		 * Calls super-constructor
		 * @param file action configuration XML file
		 * @param contextID the context's ID this action belongs to
		 */
		public DbActionLoader(File file, Object contextID) {
			super(file, contextID, DbOutputterDescriptor.class, ControllerActionConstants.OUTPUTTER_ROOT);
		}

		/**
		 * @see repast.simphony.scenario.ObjectActionLoader#createAction(java.lang.Object)
		 */
		@Override
		protected ControllerAction createAction(DbOutputterDescriptor descriptor) {
			return new DbOutputterComponentControllerAction<DbOutputterDescriptor>(descriptor);
		}

		/**
		 * @see repast.simphony.scenario.ObjectActionLoader#getClassLoader()
		 */
		@Override
		protected ClassLoader getClassLoader() {
			return getClass().getClassLoader();
		}
	}

	/**
	 * @see repast.simphony.scenario.ControllerActionIO#getActionLoader(java.io.File, java.lang.Object)
	 */
	@Override
	public ActionLoader getActionLoader(File actionFile, Object contextID) {
		return new DbActionLoader(actionFile, contextID);
	}
}
