/**
 * This file is part of
 * 
 * Repast Simphony MySQL Database Outputter
 *
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter.engine;


/**
 * repast.simphony.data.db
 * 
 * @author Sascha Holzhauer
 * @date 13.04.2010
 * 
 */
public class DbOutputterDescriptor extends AbstractDbOutputterDescriptor {

	// other fields are displayed in step 1 when the descriptor has not be initialised..
	
	/**
	 * The default driver string the text field is initially filled with:
	 */
	protected final String 		DEFAULT_DRIVER 		= "com.mysql.jdbc.Driver";
	
	/**
	 * The default URL string the text field is initially filled with:
	 */
	protected final String 		DEFAULT_URL 		= "jdbc:mysql://<mysql-server>:3306/<database>";
	
	/**
	 * True if DB login info (user-name, password shall be stored
	 */
	protected final boolean     DEFAULT_LOGIN_STORE = false;

	
	
	/**
	 * database URL
	 */
	protected String	url					= DEFAULT_URL;
	
	/**
	 * DB user name
	 */
	protected String	userName			= null;
	
	/**
	 * DB password
	 */
	protected String	password			= null;
	
	/**
	 * DB driver name
	 */
	protected String	driverName			= DEFAULT_DRIVER;
	
	/**
	 * table to store data in
	 */
	protected String	tablename			= "";

	/**
	 * table to store run information
	 */
	protected String	runInfoTablename	= "";

	/**
	 * add run id as column?
	 */
	protected boolean	addRunID			= false;
	
	/**
	 * store login information (user-name and password)?
	 */
	protected boolean	isStoringLoginInfo	= true;
	
	/**
	 * Write cache interval
	 */
	protected int		writeCacheInterval	= 0;


	/**
	 * Passes null as name.
	 */
	public DbOutputterDescriptor() {
		this(null);
	}

	/**
	 * @param name the descriptor's name
	 */
	public DbOutputterDescriptor(String name) {
		super(name);
//		super.setOutputterFactory(new SimpleFactory<DbOutputter>() {
//			
//			public DbOutputter create() {
//				return createDbOutputter();
//			}
//		});
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#getDbURL()
	 */
	
	public String getDbURL() {
		return url;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#getDriverName()
	 */
	
	public String getDriverName() {
		return driverName;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#getPassword()
	 */
	
	public String getPassword() {
		return password;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#getTableName()
	 */
	
	public String getTableName() {
		return tablename;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#getUsername()
	 */
	
	public String getUsername() {
		return userName;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#setDbURL(java.lang.String)
	 */
	
	public void setDbURL(String url) {
		this.url = url;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#setDriverName(java.lang.String)
	 */
	
	public void setDriverName(String driverName) {
		this.driverName = driverName;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#setPassword(java.lang.String)
	 */
	
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#setTableName(java.lang.String)
	 */
	
	public void setTableName(String tableName) {
		this.tablename = tableName;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#setUsername(java.lang.String)
	 */
	
	public void setUsername(String username) {
		this.userName = username;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#getWriteCacheInterval()
	 */
	
	public int getWriteCacheInterval() {
		return this.writeCacheInterval;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#setWriteCacheInterval(int)
	 */
	
	public void setWriteCacheInterval(int interval) {
		this.writeCacheInterval = interval;
	}

	/**
	 * @return the addRunID
	 */
	
	public boolean isAddRunID() {
		return addRunID;
	}

	/**
	 * @param addRunID the addRunID to set
	 */
	
	public void setAddRunID(boolean addRunID) {
		this.addRunID = addRunID;
	}
	
	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#isStoringLoginInfo()
	 */
	
	public boolean isStoringLoginInfo() {
		return isStoringLoginInfo;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#setStoringLoginInfo(boolean)
	 */
	
	public void setStoringLoginInfo(boolean storeLogin) {
		this.isStoringLoginInfo = storeLogin;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#getRunInfoTableName()
	 */
	
	public String getRunInfoTableName() {
		return runInfoTablename;
	}

	/**
	 * @see repast.simphony.data.db.outputter.engine.DbOutputterDescriptor#setRunInfoTableName(java.lang.String)
	 */
	
	public void setRunInfoTableName(String name) {
		this.runInfoTablename = name;
	}
}
