/**
 * This file is part of
 * 
 * Repast Simphony MySQL Database Outputter
 *
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter.gui;



import java.awt.Component;
import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.SwingUtilities;

import repast.simphony.data.db.outputter.engine.DbOutputterComponentControllerAction;
import repast.simphony.data.db.outputter.engine.DbOutputterDescriptor;
import repast.simphony.data2.engine.DataSetDescriptor;
import repast.simphony.data2.util.DataUtilities;
import repast.simphony.engine.controller.ControllerActionConstants;
import repast.simphony.engine.environment.ControllerAction;
import repast.simphony.engine.environment.ControllerRegistry;
import repast.simphony.ui.plugin.AbstractEditorMenuItem;



/**
 * repast.simphony.data.db
 * 
 * The new menu item of scenario tree leave "Outputters"
 * 
 * @author Sascha Holzhauer
 * @date 13.04.2010
 * 
 */
public class DbOutputterMenuItem extends AbstractEditorMenuItem {
	private static final long	serialVersionUID	= 1L;

	/**
	 * Instantiated the Menu item to create a database outputter action. Calls super constructor.
	 */
	public DbOutputterMenuItem() {
		super("Add DB Outputter");
	}

	private DbOutputterComponentControllerAction<DbOutputterDescriptor> createAction(Component parent, 
			List<Class<?>> agentClasses, 
		      List<DataSetDescriptor> dataSets) {
		
		DbOutputterWizard wizard = new DbOutputterWizard(dataSets);
	    wizard.showDialog(parent, "DB Outputter Editor");
	    if (wizard.wasCancelled()) return null;

	    DbOutputterComponentControllerAction<DbOutputterDescriptor> action = 
	    	new DbOutputterComponentControllerAction<DbOutputterDescriptor>(wizard.getModel().getDescriptor());
	    return action;
	}

	/**
	 * Invoked when an action occurs (i.e. then the menu item is clicked).
	 * Creates a {@link DbOutputterComponentControllerAction}, shows the wizard and adds to action to registry and scenario tree.
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		
		 Object contextID = evt.getContextID();
		    List<Class<?>> clazzes = evt.getScenario().getContext().getAgentClasses(true);
		    DbOutputterComponentControllerAction<DbOutputterDescriptor> action = createAction(SwingUtilities.getWindowAncestor(evt.getTree()),
		            clazzes, DataUtilities.getDataSetDescriptors(evt.getRegistry(), evt.getContextID()));
		    if (action != null) {
		      ControllerRegistry registry = evt.getRegistry();

		      ControllerAction parent = registry.findAction(contextID, ControllerActionConstants.OUTPUTTER_ROOT);
		      registry.addAction(contextID, parent, action);
		      evt.addActionToTree(action);
		    }
	}
}
