/**
 * This file is part of
 * 
 * Repast Simphony MySQL Database Outputter
 *
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter.gui;

import java.awt.Component;
import java.util.ArrayList;
import java.util.List;

import org.pietschy.wizard.PanelWizardStep;
import org.pietschy.wizard.Wizard;

import repast.simphony.data.db.outputter.engine.DbOutputterDescriptor;
import repast.simphony.data2.engine.DataSetDescriptor;
import repast.simphony.data2.wizard.SinkDataStep;

/**
 * repast.simphony.data.db
 * 
 * The DB Outputter wizard
 * 
 * @author Sascha Holzhauer
 * @date 13.04.2010
 * 
 */
public class DbOutputterWizard {
	
	/**
	 * the outputter's model
	 */
	protected DbOutputterModel model;

	/**
	 * the underlying wizard
	 */
	protected Wizard wizard;

	/**
	 * True if the wizard was cancelled.
	 */
	protected boolean cancelled;

	  public DbOutputterWizard(List<DataSetDescriptor> dataSets) {
		    this(dataSets, null);
		  }
	  
	/**
	 * @param gatherers
	 * @param scenario
	 * @param contextID
	 */
	public DbOutputterWizard(List<DataSetDescriptor> dataSets,
			DbOutputterDescriptor descriptor) {
		super();
		if (descriptor != null) {
			model = new DbOutputterModel(dataSets, descriptor);
		} else {
			model = new DbOutputterModel(dataSets);
		}
		// cancelled = true;

		buildPath();
		wizard = new Wizard(model);
		wizard.setOverviewVisible(false);
	    wizard.setDefaultExitMode(Wizard.EXIT_ON_FINISH);
	    model.setLastVisible(false);
	    
		// wizard.addWizardListener(this);
	}

	  public List<PanelWizardStep> getStepsForEditor() {
		    List<PanelWizardStep> steps = new ArrayList<PanelWizardStep>();
		    steps.add(new SinkDataStep<DbOutputterDescriptor, DbOutputterModel>("Db DataProperties", ""));
		    steps.add(new DbDataStep());
		    steps.add(new DbPropertiesStep());
		   
		    return steps;
		  }
	  
	/**
	 * Build up the wizard's path.
	 * @return the wizard's path
	 */
	private void buildPath() {
	    model.add(new SinkDataStep<DbOutputterDescriptor, DbOutputterModel>( "DB Data Properties",
        "Please enter the data set, and the data sources to record as output."));
	    model.add(new DbDataStep());
    model.add(new DbPropertiesStep());
	}

	/**
	 * @param component the parental component
	 * @param title the wizard's title
	 */
	public void showDialog(Component component, String title) {
		wizard.showInDialog(title, component, true);
	}

	/**
	 * @return the {@link DbOutputterModel}
	 */
	public DbOutputterModel getModel() {
		return model;
	}


	public boolean wasCancelled() {
	    if (wizard == null) {
	        return true;
	      } else {
	        return wizard.wasCanceled();
	      }
	}

	/**
	 * @return the wizard
	 */
	public Wizard getWizard() {
		return wizard;
	}
}
